<?php
/**
 * @file
 * Provides a helper class to generate random content.
 */

namespace Drupal\bootstrap_install;

use Drupal\Component\Utility\Random;
use Drupal\Core\File\FileSystemInterface;

use Drupal\paragraphs\ParagraphInterface;


/**
 * Provides a helper class to generate random content.
 */
class GenerateContent {

  /**
   * The random data generator.
   *
   * @var \Drupal\Component\Utility\Random
   */
  protected $random;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Profile path
   */
  protected $profilePath;

  /**
   * Get the profile path.
   */
  protected function getProfilePath() {
    if (!$this->profilePath) {
      $this->profilePath = drupal_get_path('profile', 'bootstrap_install');
    }
    return $this->profilePath;
  }

  /**
   * Gets the entity type manager service.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager service.
   */
  protected function getEntityTypeManager() {
    if (!$this->entityTypeManager) {
      $this->entityTypeManager = \Drupal::entityTypeManager();
    }
    return $this->entityTypeManager;
  }

  /**
   * Returns the random data generator.
   *
   * @return \Drupal\Component\Utility\Random
   *   The random data generator.
   */
  protected function getRandom() {
    if (!$this->random) {
      $this->random = new Random();
    }
    return $this->random;
  }

  /**
   * Generates a random sentence of specific length.
   *
   * Words are randomly selected with length from 2 up to the optional parameter
   * $max_word_length. The first word is capitalised. No ending period is added.
   *
   * @param int $sentence_length
   *   The total length of the sentence, including the word-separating spaces.
   * @param int $max_word_length
   *   (optional) Maximum length of each word. Defaults to 8.
   *
   * @return string
   *   A sentence of the required length.
   */
  protected function randomSentenceOfLength($sentence_length, $max_word_length = 8) {
    // Maximum word length cannot be longer than the sentence length.
    $max_word_length = min($sentence_length, $max_word_length);
    $words = [];
    $remainder = $sentence_length;
    do {
      if ($remainder <= $max_word_length) {
        // If near enough to the end then generate the exact length word to fit.
        $next_word = $remainder;
      }
      else {
        // Cannot fill the remaining space with one word, so choose a random
        // length, short enough for a following word of at least minimum length.
        $next_word = mt_rand(2, min($max_word_length, $remainder - 3));
      }
      $words[] = $this->getRandom()->word($next_word);
      $remainder = $remainder - $next_word - 1;
    } while ($remainder > 0);
    $sentence = ucfirst(implode(' ', $words));
    return $sentence;
  }

  /**
   * Generates a random image file.
   */
  protected function randomImage() {
    $i = rand(1, 9);
    $file_data = file_get_contents($this->getProfilePath() . '/content/images/' . $i . '.jpg');
    $file = file_save_data($file_data, 'public://' . $this->getRandom()->name(8, TRUE) . '.jpg', FileSystemInterface::EXISTS_REPLACE);
    return $file;
  }

  /**
   * Generates a random image as a Media object.
   * 
   * @return \Drupal\media\MediaInterface The image media entity.
   */
  protected function randomMediaImage() {
    $file = $this->randomImage();

    // Create the media object
    $media = $this->getEntityTypeManager()->getStorage('media')->create([
      'bundle' => 'image',
      'uid' => '1',
      'field_media_image' => [
        'target_id' => $file->id(),
        'alt' => $this->getRandom()->sentences(5),
      ],
    ]);

    $media->setName($this->getRandom()->name())
      ->setPublished(TRUE)
      ->save();

    return $media;
  }

  /**
   * Create a paragraph entity.
   * 
   * @param string $type
   *   The paragraph bundle to create.
   * @param mixed[] $fields
   *   An array of paragraph fields.
   * 
   * @return \Drupal\paragraphs\ParagraphInterface
   */
  protected function createParagraph($type, $fields) {
    $fields['type'] = $type;
    $paragraph = $this->getEntityTypeManager()
      ->getStorage('paragraph')
      ->create($fields);
    $paragraph->save();
    return $paragraph;
  }

  /**
   * Create a container paragraph entity with provided elements.
   * 
   * @param \Drupal\paragraphs\ParagraphInterface[] $elements
   *  The elements inside the container.
   * @param mixed[] $override_fields
   *  (optional) An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\ParagraphInterface
   */
  public function createContainerParagraph($elements, $override_fields = []) {
    $fields = [
      'mbp_container' => [
        'value' => 'container',
      ],
      'mbp_elements' => [],
    ];
    $fields = array_merge($fields, $override_fields);

    foreach ($elements as $element) {
      $fields['mbp_elements'][] = [
        'target_id' => $element->id(),
        'target_revision_id' => $element->getRevisionId(),
      ];
    }
    return $this->createParagraph('mbp_container', $fields);
  }

  /**
   * Create a columns equal paragraph entity with default elements.
   * 
   * @param \Drupal\paragraphs\ParagraphInterface[] $elements
   *  (optional) The elements inside each column otherwise simple text elements
   *  defaults will be generated.
   * @param integer $num_cols
   *  (optional) The number of columns to generate. Defautl is 3.
   * @param mixed[] $override_fields
   *  (optional) An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\ParagraphInterface
   */
  public function createColumnsEqualParagraph($elements = [], $num_cols = 3, $override_fields = []) {
    $fields = [
      'mbp_breakpoint' => [
        'value' => 'md',
      ],
      'mbp_elements' => [],
    ];
    $fields = array_merge($fields, $override_fields);

    if (!empty($elements)) {
      foreach ($elements as $element) {
        $fields['mbp_elements'][] = [
          'target_id' => $element->id(),
          'target_revision_id' => $element->getRevisionId(),
        ];
      }
    }
    else {
      for ($i = 0; $i < $num_cols; $i++) {
        $text = $this->createTextParagraph();
        $fields['mbp_elements'][] = [
          'target_id' => $text->id(),
          'target_revision_id' => $text->getRevisionId(),
        ];
      }
    }
    return $this->createParagraph('mbp_columns', $fields);
  }

  /**
   * Create a columns (two uneven) paragraph entity with default elements.
   * 
   * @param \Drupal\paragraphs\ParagraphInterface[] $elements
   *  (optional) The elements inside each column otherwise simple text elements
   *  defaults will be generated. Max 2. More than 2 will be ignored.
   * @param mixed[] $override_fields
   *  An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\ParagraphInterface
   */
  public function createColumnsTwoParagraph($elements = [], $override_fields = []) {
    $fields = [
      'mbp_breakpoint' => [
        'value' => 'md',
      ],
      'mbp_layout_2' => [
        'value' => 'mbp-layout--25-75',
      ],
      'mbp_elements_2' => [],
    ];
    $fields = array_merge($fields, $override_fields);

    if (!empty($elements)) {
      for ($i = 0; $i < 2; $i++) {
        $fields['mbp_elements_2'][] = [
          'target_id' => $elements[$i]->id(),
          'target_revision_id' => $elements[$i]->getRevisionId(),
        ];
      }
    }
    else {
      for ($i = 0; $i < 2; $i++) {
        $text = $this->createTextParagraph();
        $fields['mbp_elements_2'][] = [
          'target_id' => $text->id(),
          'target_revision_id' => $text->getRevisionId(),
        ];
      }
    }
    return $this->createParagraph('mbp_columns_2', $fields);
  }

  /**
   * Create a columns (three uneven) paragraph entity with default elements.
   * 
   * @param \Drupal\paragraphs\ParagraphInterface[] $elements
   *  (optional) The elements inside each column otherwise simple text elements
   *  defaults will be generated. Max 3. More than 3 will be ignored.
   * @param mixed[] $override_fields
   *  An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\ParagraphInterface
   */
  public function createColumnsThreeParagraph($elements = [], $override_fields = []) {
    $fields = [
      'mbp_breakpoint' => [
        'value' => 'md',
      ],
      'mbp_layout_3' => [
        'value' => 'mbp-layout--25-50-25',
      ],
      'mbp_elements_3' => [],
    ];
    $fields = array_merge($fields, $override_fields);

    if (!empty($elements)) {
      for ($i = 0; $i < 3; $i++) {
        $fields['mbp_elements_3'][] = [
          'target_id' => $elements[$i]->id(),
          'target_revision_id' => $elements[$i]->getRevisionId(),
        ];
      }
    }
    else {
      for ($i = 0; $i < 3; $i++) {
        $text = $this->createTextParagraph();
        $fields['mbp_elements_3'][] = [
          'target_id' => $text->id(),
          'target_revision_id' => $text->getRevisionId(),
        ];
      }
    }
    return $this->createParagraph('mbp_columns_3', $fields);
  }

  /**
   * Create a new text paragraph entity with random content.
   * 
   * @param string $content
   *   (optional) The text content for the paragraph or empty string to 
   *   generate a random sentance.
   * @param int $sentence_length
   *   (optional) The total length of the sentence, including the 
   *   word-separating spaces. Default is 300.
   * @param int $format
   *   (optional) The text format to use. Default is 'rich_text'.
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createTextParagraph($content = NULL, $sentence_length = '300', $format = 'rich_text') {
    $fields = [
      'mp_text' => [
        'value' => $content ?? '<p>' . $this->randomSentenceOfLength($sentence_length) . '</p>',
        'format' => $format,
      ]
    ];
    return $this->createParagraph('mp_text', $fields);
  }

  /**
   * Create a new text paragraph from the contents of an HTML file.
   * 
   * @param int $file_path
   *   The path to an HTML file. 
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createTextParagraphFromFile($file_path) {
    return $this->createTextParagraph(file_get_contents($file_path));
  }

  /**
   * Create a new card paragraph entity with random content.
   * 
   * @param string $title
   *   (optional) The title text the card or NULL to generate a random title.
   * @param string $content
   *   (optional) The text content for the paragraph or NULL to generate a 
   *   random sentance.
   * @param int $sentence_length
   *   (optional) The total length of the sentence, including the 
   *   word-separating spaces. Default is 200.
   * @param int $format
   *   (optional) The text format to use. Default is 'rich_text'.
   * @param mixed[] $override_fields
   *  An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createCardParagraph($title = NULL, $content = NULL, $sentence_length = '200', $format = 'rich_text', $override_fields = []) {
    $image = $this->randomMediaImage();

    $fields = [
      'mbp_card_image' => [
        'target_id' => $image->id(),
      ],
      'mp_title' => [
        'value' => $content ?? $this->randomSentenceOfLength(25),
      ],
      'mp_text' => [
        'value' => $content ?? '<p>' . $this->randomSentenceOfLength($sentence_length) . '</p>',
        'format' => $format,
      ],
      'mbp_cta' => [
        'uri' => 'https://www.example.com',
        'title' => 'Call to Action',
      ],
      'mp_title_size' => [
        'value' => 'h2',
      ],
    ];
    $fields = array_merge($fields, $override_fields);
    return $this->createParagraph('mbp_card', $fields);
  }

  /**
   * Create a new accordion paragraph entity with random content.
   * 
   * @param string $num_sections
   *   (optional) The number of accordion sections to generate.
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createAccordionParagraph($num_sections) {
    $fields = [];
    for ($i = 0; $i < $num_sections; $i++) {
      $text = $this->createTextParagraph();
      $section_fields = [
        'mbp_accordion_open' => [
          'value' => ($i == 0) ? 1 : 0,
        ],
        'mbp_elements' => [[
          'target_id' => $text->id(),
          'target_revision_id' => $text->getRevisionId(),
        ]],
        'mp_title_size' => [
          'value' => 'h3',
        ],
        'mp_title' => [
          'value' => $this->randomSentenceOfLength(25),
        ],
      ];
      $section = $this->createParagraph('mbp_accordion_section', $section_fields);
      $fields['mbp_accordion_sections'][] = [
        'target_id' => $section->id(),
        'target_revision_id' => $section->getRevisionId(),
      ];
    }
    return $this->createParagraph('mbp_accordion', $fields);
  }

  /**
   * Create a new tab paragraph entity with random content.
   * 
   * @param string $num_sections
   *  The number of tab sections to generate.
   * @param mixed[] $override_fields
   *  An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createTabParagraph($num_sections, $override_fields = []) {
    $fields = [];
    for ($i = 0; $i < $num_sections; $i++) {
      $text = $this->createTextParagraph(NULL, 600);
      $section_fields = [
        'mbp_elements' => [[
          'target_id' => $text->id(),
          'target_revision_id' => $text->getRevisionId(),
        ]],
        'mp_title' => [
          'value' => $this->randomSentenceOfLength(10),
        ],
      ];
      $section = $this->createParagraph('mbp_tab_section', $section_fields);
      $fields['mbp_tab_sections'][] = [
        'target_id' => $section->id(),
        'target_revision_id' => $section->getRevisionId(),
      ];
    }
    $fields = array_merge($fields, $override_fields);
    return $this->createParagraph('mbp_tab', $fields);
  }

  /**
   * Create a new carousel paragraph entity with random content.
   * 
   * @param string $num_slides
   *  The number of carousel slides to generate.
   * @param mixed[] $override_fields
   *  An array of fields to override the default generated values.
   * 
   * @return \Drupal\paragraphs\Entity\ParagraphInterface
   */
  public function createCarouselParagraph($num_slides, $override_fields = []) {
    $fields = [];
    for ($i = 0; $i < $num_slides; $i++) {
      $image = $this->randomImage();
      $slide_fields = [
        'mbp_slide_image' => [
          'target_id' => $image->id(),  
          'alt' => $this->getRandom()->sentences(5),
        ],
        'mp_text' => [
          'value' => $this->randomSentenceOfLength(150),
        ],
      ];
      $slide = $this->createParagraph('mbp_carousel_slide', $slide_fields);
      $fields['mbp_slides'][] = [
        'target_id' => $slide->id(),
        'target_revision_id' => $slide->getRevisionId(),
      ];
    }
    $fields = array_merge($fields, $override_fields);
    return $this->createParagraph('mbp_carousel', $fields);
  }

}
